-- GridStatusCombatLogHealing.lua
--
-- Created By : Dagma
-- Based on GridStatusCombatLogHealing by Pachelbel

-- Libraries
local RL = AceLibrary("Roster-2.1")
local L = AceLibrary("AceLocale-2.2"):new("GridStatusCombatLogHealing")

-- Module
GridStatusCombatLogHealing = GridStatus:NewModule("GridStatusCombatLogHealing", "AceHook-2.1")

local GridStatusCombatLogHealing = GridStatusCombatLogHealing
GridStatusCombatLogHealing.menuName = L["Combat Log Healing"]
GridStatusCombatLogHealing.options = false

GridStatusCombatLogHealing.defaultDB = {
    debug = false,
    enable = true,
    version = "0.2",

    combatloghealing = {
        text = L["Healing"],
        enable = true,
        color = { r = 1.0, g = 0.65, b = 0.0, a = 1.0 },  -- hopefully light orange
        priority = 49,  -- the default incoming heal priority is 50, so we want default to below that
        duration = 2.0,  -- how long they should get the dmg indicator for (seconds)
        threshold = 500,  -- how big of a hit does it have to be?
        others = true, -- show healing from sources other than player?
        range = false,
    },
}

local combatloghealingOptions = {
    ["showothers"] = {
        type = 'toggle',
        name = "Others",
        desc = "Show healing from other players.",
        get = function()
            return GridStatusCombatLogHealing.db.profile.combatloghealing.others
        end,
        set = function(v)
            GridStatusCombatLogHealing.db.profile.combatloghealing.others = v
        end,
    },
    ["duration"] = {
        type = 'range',
        name = L["Indicator Duration"],
        desc = L["Seconds healing indicator should remain on"],
        get = function() 
            return GridStatusCombatLogHealing.db.profile.combatloghealing.duration 
        end,
        set = function(v)
            GridStatusCombatLogHealing.db.profile.combatloghealing.duration = v
        end,
        min = 1.0,
        max = 5.0,
        step = 0.5,
        isPercent = false,
    },
    ["threshold"] = {
        type = 'range',
        name = L["Healing Threshold"],
        desc = L["Amount healing has to be to trigger indicator"],
        get = function() 
            return GridStatusCombatLogHealing.db.profile.combatloghealing.threshold
        end,
        set = function(v)
            GridStatusCombatLogHealing.db.profile.combatloghealing.threshold = v
        end,
        min = 0,
        max = 10000,
        step = 100,
        isPercent = false,
    },
}

function GridStatusCombatLogHealing:OnInitialize()
    self.super.OnInitialize(self)
    self:RegisterStatus("combatloghealing", L["Combat Log Healing"], combatloghealingOptions, true)

    --!!! once enable is fixed remove below
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    -- trace out event
   -- DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:OnInitialize")
end

function GridStatusCombatLogHealing:OnEnable()
    self:UnregisterAllEvents()
    
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    -- trace out event
    --DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:OnEnable")
end

function GridStatusCombatLogHealing:OnDisable()
    self:UnregisterAllEvents()

    -- trace out event
    --DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:OnDisable")
end

function GridStatusCombatLogHealing:FormatDamageText(dmg)
    local dmgText
    if ((dmg < -999) or (dmg>999)) then
        dmgText = string.format("%.1fk", dmg/1000.0)
    else
        dmgText = string.format("%d", dmg)
    end
    return dmgText
end

-- some "avg" stats on these
--26271 SPELL_PERIODIC_HEAL
--13853 SPELL_DAMAGE
-- 9244 SWING_DAMAGE
--  569 RANGE_DAMAGE
--    1 ENVIRONMENTAL_DAMAGE
--13271 SPELL_PERIODIC_DAMAGE
-- 9511 SPELL_HEAL
--  366 DAMAGE_SHIELD
--    0 DAMAGE_SPLIT -- this is used, but i have no stats on it
-- these appear to be unused
--SPELL_DRAIN
--SPELL_LEECH
--SPELL_PERIODIC_DRAIN
--SPELL_PERIODIC_LEECH

--SPELL_PERIODIC_DAMAGE,0x0000000000000000,nil,0x80000000,0x0000000000467BD9,"Uberpriest",0x514,32409,"Shadow Word: Death",0x20,1804,32,0,0,0,nil,nil,nil
--SPELL_ENERGIZE,0x0000000001F903D6,"Uberlock",0x514,0x0000000001F903D6,"Uberlock",0x514,31818,"Life Tap",0x20,1863,0
--msg1=spellId (31818|32553), msg2=spellName, msg3=spellSchool, msg4=amount,
--SPELL_HEAL,0x0000000001F903D6,"Uberlock",0x514,0x0000000001F903D6,"Uberlock",0x514,27223,"Death Coil",0x20,1426,nil
-- Death Coil (6789, 17625, 17926, 27223)
-- Siphon Life (18265, 18879, 18880, 18881, 27264, 30911)
-- Drain Life (689, 699, 709, 7651, 11699, 11700, 27219, 27220)
-- Unstable Affliction (30108, 30404, 30405)

function GridStatusCombatLogHealing:CombatLogFilter(timestamp, msg, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, msg1, msg2, msg3, msg4)
    local prefix, suffix, suffix2 = strsplit("_", msg)
    
    if suffix2 == nil then suffix2 = "." end
    
    -- these are ordered so the ones we see most often are checked first
    --if msg == "SPELL_PERIODIC_HEAL" then
    --    return srcName,msg4
    --end
    --if suffix == "DAMAGE" then
    --    if prefix == "SPELL" then
    --        return -msg4
    --    elseif prefix == "SWING" then
    --        return -msg1
    --    elseif prefix == "RANGE" then
    --        return -msg4
    --    elseif prefix == "ENVIRONMENTAL" then
    --        return -msg2
    --    end
    --end
    --if msg == "SPELL_PERIODIC_DAMAGE" then
    --    return -msg4
    --end
    if suffix == "HEAL" then
        --return msg4
        return srcName,msg4,dstName
    end
    --if msg == "DAMAGE_SHIELD" then
    --    return -msg4
    --end
    --if msg == "DAMAGE_SPLIT" then
    --    return -msg4
    --end
    --if suffix == "ENERGIZE" then
    --    if prefix == "SPELL" then
    --        if ((msg1 == 31818) or (msg1 == 32553)) then
    --            -- warlock life tap, not exact, but better than nothing
    --            return -msg4
    --        end
    --    end
    --end
    
    -- new spell_start event signalling
    -- will split this off to another add-on if it works
    --if suffix2 == "START" then
        --DEFAULT_CHAT_FRAME:AddMessage( msg2 )
    --    return srcName,2000,srcName
    --end
    
    return "",0
end

-- Converts the full name (which is the name of the unit together with 
-- its realm) to the grid name ("Roster-2.1" name actually) which is 
-- simply the name of the unit
function GridStatusCombatLogHealing:GetGridName(fullName)
    if (fullName) then return fullName:match("(.+)%-") or fullName end
    return fullName
end

function GridStatusCombatLogHealing:LoseStatus(name, settings, gridName)
    -- cancel any previous events to clear Dmg indicator
    if (self:CancelScheduledEvent("ClearCombatLogHealing_" .. name)) then
        -- if event canceled, set status as lost
        self.core:SendStatusLost(gridName, "combatloghealing")
    end

    -- trace out event
    --DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:LoseStatus " .. name)
end

function GridStatusCombatLogHealing:GainStatus(name, settings, gridName)
    -- cancel any previous events to clear Dmg indicator
    self:CancelScheduledEvent("ClearCombatLogHealing_" .. name)

    self.core:SendStatusGained(gridName,
                                "combatloghealing",
                                settings.priority,
                                nil,
                                settings.color,
                                settings.text,
                                nil,
                                nil,
                                settings.icon)

    -- add event to remove Dmg indicator in duration seconds
    self:ScheduleEvent("ClearCombatLogHealing_" .. name, self.LoseStatus, settings.duration, self, name, settings, gridName)

    -- trace out event
    --DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:GainStatus " .. name)
end

function GridStatusCombatLogHealing:COMBAT_LOG_EVENT_UNFILTERED(...)

    local settings = self.db.profile.combatloghealing
    -- if not enabled then ignore
    
    --if settings.enable then DEFAULT_CHAT_FRAME:AddMessage( "enabled" ) end
    
    if (not settings.enable) then return end

    -- in my tests, around 40% of messages were healthDeltas
    local srcName,healthDelta,disName = self:CombatLogFilter(...)
    
    -- if not a healthDelta then ignore
    if (healthDelta == 0) then return end
    
    -- if not player and not others then ignore
    --DEFAULT_CHAT_FRAME:AddMessage( srcName )
    if ( UnitName("player") ~= srcName ) then
        if ( not settings.others ) then
            return
        end
    end
    
    -- in my tests, with a threshold of 500, roughly 40% of deltas were larger
    -- if healthDelta is not bigger than threshold value then ignore
    if ( healthDelta<=settings.threshold ) then return end
    
    -- in my tests, around 60% of the messages were for party/raid members
    local dstName = select(7, ...)
    local gridName = self:GetGridName(disName) -- strip realm if present
    -- if unit is not in the raid then ignore
    local u = RL:GetUnitObjectFromName(gridName)
    if not u then return end
    
    -- trace out event
    --DEFAULT_CHAT_FRAME:AddMessage("GridStatusCombatLogHealing:COMBAT_LOG_EVENT_UNFILTERED " .. dstName .. " " .. healthDelta)

    -- in my tests, dmg/heal ratio seems about 50%
    if (healthDelta < 0) then
        -- gain status for target
        --settings.text = self:FormatDamageText(healthDelta)
        --self:GainStatus(dstName, settings, gridName)
    else
        -- lose status for target
        --settings.text = self:FormatDamageText(healthDelta)
        settings.text = srcName
        local thespellid = select(9,...)
        local _, _, icon = GetSpellInfo( thespellid ) -- get spellid and then texture from it
        settings.icon = icon
        self:GainStatus(disName, settings, gridName)
        --self:LoseStatus(dstName, settings, gridName)
    end
end
